ITEMDB_DBFRAME_VERSION = "1.1 2006/12/20";
ITEMDB_DBFRAME_VERSION_NUMERIC = 20061220001;

-- --------------------
-- --------------------
-- these should all be nil for release
ITEMDB_DISTCHAT_DISABLE = nil;
ITEMDB_MOUSEOVERINSPECT_DISABLE = nil;

ITEMDB_DEBUG_ENABLE = nil;
ITEMDB_DEBUG_INSPECT = nil;
ITEMDB_DEBUG_SESSIONITEMS = nil;
-- --------------------
-- --------------------

--[[
ITEM_QUALITY_HEXCOLORS = {
	[0] = "|cff9d9d9d",
	[1] = "|cffffffff",
	[2] = "|cff1eff00",
	[3] = "|cff0070dd",
	[4] = "|cffa335ee",
	[5] = "|cffff8000",
	[6] = "|cffe6cc80",
} ]]

ITEM_QUALITY_HEXCOLORS = { };
for i = -1, 6 do
	_, _, _, ITEM_QUALITY_HEXCOLORS[i] = GetItemQualityColor(i);
end

ItemDB_InventorySlotName = {
	"HeadSlot",
	"NeckSlot",
	"ShoulderSlot",
	"BackSlot",
	"ChestSlot",
	"ShirtSlot",
	"TabardSlot",
	"WristSlot",
	"HandsSlot",
	"WaistSlot",
	"LegsSlot",
	"FeetSlot",
	"Finger0Slot",
	"Finger1Slot",
	"Trinket0Slot",
	"Trinket1Slot",
	"MainHandSlot",
	"SecondaryHandSlot",
	"RangedSlot",
	"AmmoSlot",
	"Bag0Slot",
	"Bag1Slot",
	"Bag2Slot",
	"Bag3Slot",
}

local DressUpItem_orig = DressUpItem;

function DressUpItem(item)
	if ( ItemDB_DBFrame:IsVisible() ) then
		if ( not ItemDB_DBFrame_DressUpFrame:IsVisible() ) then
			ShowUIPanel(ItemDB_DBFrame_DressUpFrame);
			ItemDB_DBFrame_DressUpModel:SetUnit("player");
		end
		ItemDB_DBFrame_DressUpModel:TryOn(item);
	else
		DressUpItem_orig(item);
	end
end

function Set_ItemDB_DBFrame_DressUpBackground()
	local texture = DressUpTexturePath();
	ItemDB_DBFrame_DressUpBackgroundTop:SetTexture(texture..1);
	ItemDB_DBFrame_DressUpBackgroundBot:SetTexture(texture..3);
end

local ItemDBRarityFilter = {1, 1, 1, 1, 1, 1, 1};
local ItemDBSortReverse = 0;
local ItemDBSortCurrent = "name";
local ItemDB_DBFrame_selectedClass;
local ItemDB_DBFrame_selectedSubclass;
local ItemDB_DBFrame_selectedInvtype;
local ItemDB_DBFrame_selectedTab;
local ItemDB_DBFrame_clickedButton;

ITEMDB_NUM_ITEMS_TO_DISPLAY = 8;
ITEMDB_NUM_FILTERS_TO_DISPLAY = 15;
ITEMDB_ITEM_FILTER_HEIGHT = 20;
ITEMDB_ITEM_BUTTON_HEIGHT = 37;
ITEMDB_CLASS_FILTERS = {};
ITEMDB_OPEN_FILTER_LIST = {};

local ItemsAll;
local ItemsSession;
local ItemsSessionKnown;
local ItemsSessionRecheck;
local ItemsList;
local types;
local ItemTypes;
local ItemSubTypes;
local ItemInvTypes;

local ItemDB_DistChat_Base = "ItemDBDistChat";
local ItemDB_DistChat_Name;
local ItemDB_DistChat_ID;

local ItemDB_DistChat_msg_per_sec = 15;
local ItemDB_DistChat_msg_per_sec_offer = 3;

local ItemDB_DistChat_Queue_Offering = {};
local ItemDB_DistChat_Queue_Offers = {};
local ItemDB_DistChat_Queue_Requests = {};
local ItemDB_DistChat_PlayersInvolved = {};

local ItemDB_DistChat_last_Outgoing = time();

local ItemDB_last_FrameScan = time();
local ItemDB_last_FrameScan_blocktime = 1;
local ItemDB_last_Version = time();
local ItemDB_newest_Version = ITEMDB_DBFRAME_VERSION_NUMERIC;
local ItemDB_loadtime = time();
local ItemDB_DistChat_lastcheck = 0;
local ItemDB_last_rebuild = time();

local ItemDB_Inspected = {};
local ItemDB_Inspected_cleaned = time();

local ItemDB_ChatMessageTypes = {
	["CHAT_MSG_SYSTEM"] = "",
	["CHAT_MSG_SAY"] = "",
	["CHAT_MSG_TEXT_EMOTE"] = "",
	["CHAT_MSG_YELL"] = "",
	["CHAT_MSG_WHISPER"] = "",
	["CHAT_MSG_WHISPER_INFORM"] = "",
	["CHAT_MSG_PARTY"] = "",
	["CHAT_MSG_GUILD"] = "",
	["CHAT_MSG_OFFICER"] = "",
	["CHAT_MSG_CHANNEL"] = "",
	["CHAT_MSG_RAID"] = "",
	["CHAT_MSG"] = "",
	["CHAT_MSG_LOOT"] = "",
	["CHAT_MSG_TRADE"] = "",
};

UIPanelWindows["ItemDB_DBFrame"] = { area = "doublewide", pushable = 0, whileDead = 1 };

MoneyTypeInfo["ITEMDB"] = {
	UpdateFunc = function()
		return this.staticMoney;
	end,
	showSmallerCoins = 1,
	fixedWidth = 1,
	collapse = 1,
	truncateSmallCoins = nil,
};

StaticPopupDialogs["ITEMDB_LINKING_WARNING"] = {
	text = TEXT(ITEMDB_STRING_LINKING_WARNING),
	button1 = TEXT(YES),
	button2 = TEXT(NO),
	OnAccept = function()
		ChatEdit_InsertLink(ItemDB_DBFrame_ItemButton_GetLink(ItemDB_DBFrame_clickedButton));
	end,
	showAlert = 1,
	whileDead = 1,
	timeout = 0,
	hideOnEscape = 1
};

local ItemDB_DBFrame_original_ChatFrame_OnEvent = ChatFrame_OnEvent;

local ItemDB_ERR_CHAT_PLAYER_NOT_FOUND_S_pattern = string.gsub(ERR_CHAT_PLAYER_NOT_FOUND_S, "%%s", "%(%.%+%)");

function ChatFrame_OnEvent(event)
	if( (event == "CHAT_MSG_WHISPER" or event == "CHAT_MSG_WHISPER_INFORM") and arg1 ) then
		if (string.find(arg1, "%[ItemDB%-request%]")) then
			return;
		end
		if (string.find(arg1, "%[ItemDB%-answer%]")) then
			return;
		end
	end
	if( (event == "CHAT_MSG_AFK" or event == "CHAT_MSG_DND" or event == "CHAT_MSG_IGNORED") and ItemDB_DistChat_PlayersInvolved[arg2] and (time() - ItemDB_DistChat_PlayersInvolved[arg2]) < 15 ) then
		return;
	end
	if (event == "CHAT_MSG_SYSTEM") then
		local _, _, playername = string.find(arg1, ItemDB_ERR_CHAT_PLAYER_NOT_FOUND_S_pattern);
		if ( playername and ItemDB_DistChat_PlayersInvolved[playername] and (time() - ItemDB_DistChat_PlayersInvolved[playername]) < 5 ) then
			return;
		end
	end
	ItemDB_DBFrame_original_ChatFrame_OnEvent(event);
end

function ItemDB_DBFrame_OnLoad()
	-- Register slash command
	SLASH_ITEMDB1 = "/itemdb";
	SlashCmdList["ITEMDB"] = function(msg)
		ItemDB_DBFrame_SlashCommandHandler(msg);
	end

	PanelTemplates_SetNumTabs(this, 2);
	PanelTemplates_SetTab(ItemDB_DBFrame, 1);

	--register the chat types to get the items put in chat
	for ChatMessageType, _ in pairs(ItemDB_ChatMessageTypes) do
		this:RegisterEvent(ChatMessageType);
	end
	this:RegisterEvent("AUCTION_ITEM_LIST_UPDATE");
	this:RegisterEvent("BANKFRAME_OPENED");
	this:RegisterEvent("MERCHANT_SHOW");
	this:RegisterEvent("MERCHANT_UPDATE");
	this:RegisterEvent("TRADE_SKILL_SHOW");
	this:RegisterEvent("TRADE_SKILL_UPDATE");
	this:RegisterEvent("CRAFT_SHOW");
	this:RegisterEvent("CRAFT_UPDATE");

	this:RegisterEvent("PLAYER_TARGET_CHANGED");
	this:RegisterEvent("UPDATE_MOUSEOVER_UNIT");

	this:RegisterEvent("PLAYER_LOGOUT");

	if( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("|cffffff00"..ITEMDB_STRING_TITLE.." "..ITEMDB_STRING_LOADED.."|cffff0000"..ITEMDB_DBFRAME_VERSION.."|r");
	end
end

function ItemDB_DBFrame_OnShow()
	if (not ItemsAll) then
		ItemDB_DBFrame_BuildItemsAll();
	end
	if (not ItemsSession) then
		ItemDB_DBFrame_BuildItemsSession();
	end
	ItemDB_DBFrame_Inspect("player");
	ItemDB_DBFrame_ScanBags();
	ItemDB_DBFrame_RecheckSessionItems();
	ItemDB_DBFrame_Filters_Update()
	if (not ItemsList) then
		ItemDB_DBFrame_BuildItemsList();
	end
	if (not ItemDB_DBFrame_selectedTab) then
		ItemDB_DBFrame_Tab_OnClick(1);
	end
end

function ItemDB_DBFrame_OnEvent()
	if (event == "PLAYER_LOGOUT") then
		ItemDB_DistChat_Leave();
	elseif (event == "AUCTION_ITEM_LIST_UPDATE") then
		ItemDB_DBFrame_ScanAuction();
	elseif (event == "BANKFRAME_OPENED") then
		ItemDB_DBFrame_Inspect("player");
		ItemDB_DBFrame_ScanBags();
		ItemDB_DBFrame_ScanBank();
	elseif (event == "MERCHANT_SHOW" or event == "MERCHANT_UPDATE") then
		ItemDB_DBFrame_ScanMerchant();
	elseif (event == "TRADE_SKILL_SHOW" or event == "TRADE_SKILL_UPDATE") then
		ItemDB_DBFrame_ScanTradeSkill();
	elseif (event == "CRAFT_SHOW" or event == "CRAFT_UPDATE") then
		ItemDB_DBFrame_ScanCraft();
	elseif( event == "PLAYER_TARGET_CHANGED" ) then
		if (not UnitExists("target") or UnitIsUnit("target", "player")) then
			return;
		elseif (UnitIsPlayer("target")) then
			ItemDB_DBFrame_Inspect("target");
		end
	elseif( event == "UPDATE_MOUSEOVER_UNIT" and not ITEMDB_MOUSEOVERINSPECT_DISABLE) then
		if (not UnitExists("mouseover") or UnitIsUnit("mouseover", "player")) then
			return;
		elseif (UnitIsPlayer("mouseover")) then
			ItemDB_DBFrame_Inspect("mouseover");
		end
	elseif (ItemDB_ChatMessageTypes[event] and arg1) then
		if (not ITEMDB_DISTCHAT_DISABLE and event == "CHAT_MSG_CHANNEL" and ItemDB_DistChat_Name and arg9 == ItemDB_DistChat_Name) then
			if (string.find(arg1, "%[ItemDB%-offer%]")) then
				ItemDB_DBFrame_ProcessChatOffer(arg1, arg2);
			elseif (string.find(arg1, "%[ItemDB%-version%]")) then
				ItemDB_DBFrame_ProcessVersion(arg1);
			end
		elseif (not ITEMDB_DISTCHAT_DISABLE and event == "CHAT_MSG_WHISPER" and string.find(arg1, "%[ItemDB%-request%]")) then
			ItemDB_DBFrame_ProcessChatRequest(arg1, arg2);
		else
			ItemDB_DBFrame_ProcessChat(arg1);
		end
--		if (not ITEMDB_DISTCHAT_DISABLE and ItemDB_DistChat_Name) then
--			ItemDB_DBFrame_DoDistChat();
--		end
	end
	if (not ITEMDB_DISTCHAT_DISABLE and (event == "CHAT_MSG_CHANNEL" or time() - ItemDB_loadtime > 300) and (not ItemDB_DistChat_Name or not ItemDB_DistChat_ID or ItemDB_DistChat_ID <= 0)) then
		ItemDB_DistChat_Leave();
		ItemDB_DistChat_Join();
	end
end

function ItemDB_DBFrame_OnUpdate()
	if (not ITEMDB_DISTCHAT_DISABLE and ItemDB_DistChat_Name) then
		if (time() - ItemDB_DistChat_lastcheck > 60) then
			ItemDB_DistChat_lastcheck = time();
			ItemDB_DistChat_ID = GetChannelName(ItemDB_DistChat_Name);
			if (not (ItemDB_DistChat_ID > 0)) then
				ItemDB_DistChat_Leave();
			end
		else
			ItemDB_DBFrame_DoDistChat();
		end
	end
end

function ItemDB_DBFrame_Tab_OnClick(index)
	if ( not index ) then
		index = this:GetID();
	end
	PanelTemplates_SetTab(ItemDB_DBFrame, index);
	if (index ~= ItemDB_DBFrame_selectedTab) then
		ItemDB_DBFrame_selectedTab = index;
		ItemDB_DBFrame_BuildItemsList();
	end
	PlaySound("igCharacterInfoTab");
end

function ItemDB_DBFrame_SlashCommandHandler(msg)
	if ( not msg or msg == "" ) then
		ItemDB_DBFrame_ToggleUI();
	elseif (strlower(msg) == strlower("status")) then
		if( DEFAULT_CHAT_FRAME ) then
			if (not ItemsAll) then
				ItemDB_DBFrame_BuildItemsAll();
			end
			if (not ItemsSession) then
				ItemDB_DBFrame_BuildItemsSession();
			end
			DEFAULT_CHAT_FRAME:AddMessage("|cffffff00[ItemDB] Status:|r ".."Version = "..ITEMDB_DBFRAME_VERSION);
			if (ItemDB_DistChat_Name) then
				DEFAULT_CHAT_FRAME:AddMessage("|cffffff00[ItemDB] Status:|r ".."DistChat = "..ItemDB_DistChat_ID..". "..ItemDB_DistChat_Name);
			elseif (ITEMDB_DISTCHAT_DISABLE) then
				DEFAULT_CHAT_FRAME:AddMessage("|cffffff00[ItemDB] Status:|r ".."DistChat = |cffff0000disabled");
			else
				DEFAULT_CHAT_FRAME:AddMessage("|cffffff00[ItemDB] Status:|r ".."DistChat = |cffff0000not joined");
			end
			DEFAULT_CHAT_FRAME:AddMessage("|cffffff00[ItemDB] Status:|r ".."DistChatQueue = "..#ItemDB_DistChat_Queue_Requests.." / "..#ItemDB_DistChat_Queue_Offers.." / "..#ItemDB_DistChat_Queue_Offering);
			DEFAULT_CHAT_FRAME:AddMessage("|cffffff00[ItemDB] Status:|r ".."ItemsAll = "..#ItemsAll);
			DEFAULT_CHAT_FRAME:AddMessage("|cffffff00[ItemDB] Status:|r ".."ItemsSession = "..#ItemsSession);
		end
	elseif (strlower(msg) == strlower("hop")) then
		ItemDB_DistChat_Leave();
	elseif (strlower(msg) == strlower("join")) then
		ItemDB_DistChat_Join();
	elseif (strlower(msg) == strlower("leave")) then
		ITEMDB_DISTCHAT_DISABLE = true;
		ItemDB_DistChat_Leave();
	end
end

function ItemDB_DBFrame_ToggleUI()
	if( ItemDB_DBFrame:IsVisible() ) then
		HideUIPanel(ItemDB_DBFrame);
	else
		ShowUIPanel(ItemDB_DBFrame);
	end
end

-- public functions
ItemDB = {
	GetInfos = function(self)
		local Info = {
			Version = ITEMDB_DBFRAME_VERSION,
			ItemsSession = 0,
			ItemsAll = 0,
			ItemsPercentage = 0,
			DistChat = nil,
			DistChatRequests = 0,
			DistChatOffers = 0,
			DistChatItems = 0,
		}
		if ItemsSession then
			Info.ItemsSession = #ItemsSession
		end
		if ItemsAll then
			Info.ItemsAll = #ItemsAll
		end
		if Info.ItemsAll > 0 then
--			Info.ItemsPercentage = ceil(Info.ItemsSession / Info.ItemsAll * 1000) / 10
			Info.ItemsPercentage = string.format("%.1f%%", Info.ItemsSession / Info.ItemsAll * 100)
		end
		if (ItemDB_DistChat_Name) then
			Info.DistChat = ItemDB_DistChat_ID..". "..ItemDB_DistChat_Name
			Info.DistChatRequests = #ItemDB_DistChat_Queue_Requests
			Info.DistChatOffers = #ItemDB_DistChat_Queue_Offers
			Info.DistChatItems = #ItemDB_DistChat_Queue_Offering
		end
		return Info
	end
}

function ItemDB_DBFrame_BuildItemsAll()
-- -----
--	if( DEFAULT_CHAT_FRAME ) then
--		DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: generating item list...");
--	end
-- -----
	ItemDB_last_rebuild = time();
	ItemsAll = {};
	types = {}
	for i = 1, 40000 do	-- 40000 should suffice for new patches
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc = GetItemInfo(i);
		if ( itemName ) then	-- valid item
			itemEquipLoc = getglobal(itemEquipLoc);
			if (not itemEquipLoc or itemEquipLoc == "") then
				itemEquipLoc = ITEMDB_STRING_INVTYPE_OTHER;
			end
			table.insert(ItemsAll, i);
			if (not types[itemType]) then
				types[itemType] = { };
			end
			if (not types[itemType][itemSubType]) then
				types[itemType][itemSubType] = { };
			end
			if (not types[itemType][itemSubType][itemEquipLoc]) then
				types[itemType][itemSubType][itemEquipLoc] = "";
			end
		end
	end
-- -----
	if( ITEMDB_DEBUG_ENABLE and DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: generating item list... found "..#ItemsAll.." items in cache...");
	end
-- -----
	ItemTypes = { };
	ItemSubTypes = { };
	ItemInvTypes = { };
	for type, subtypes in pairs(types) do
		table.insert(ItemTypes, type);
		local subtypesort = { };
		local hasinvtypes = false;
		for subtype, invtypes in pairs(subtypes) do
			table.insert(subtypesort, subtype);
			local invtypesort = { };
			for invtype, _ in pairs(invtypes) do
				table.insert(invtypesort, invtype);
-- -----
--				if( DEFAULT_CHAT_FRAME ) then
--					DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: "..type.." - "..subtype.." - "..invtype);
--				end
-- -----
			end
			if (#invtypesort > 1) then
				table.sort(invtypesort);
				if (not ItemInvTypes[type]) then
					ItemInvTypes[type] = { };
				end
				ItemInvTypes[type][subtype] = invtypesort;
				hasinvtypes = true;
			end
		end
		if (#subtypesort > 1 or hasinvtypes) then
			table.sort(subtypesort);
			ItemSubTypes[type] = subtypesort;
		end
	end
	table.sort(ItemTypes);
end

function ItemDB_DBFrame_BuildItemsSession()
	ItemsSession = {};
	ItemsSessionKnown = {};
	ItemsSessionRecheck = {};
	ItemDB_DBFrame_Inspect("player");
	ItemDB_DBFrame_ScanBags();
end

function ItemDB_DBFrame_BuildItemsList()
	FauxScrollFrame_SetOffset(ItemDB_DBFrame_ItemList_BrowseScrollFrame, 0);
	local scrollbar = getglobal("ItemDB_DBFrame_ItemList_BrowseScrollFrameScrollBar");
	scrollbar:SetValue(0);
	if (time() - ItemDB_last_rebuild > 300) then
		ItemDB_DBFrame_BuildItemsAll();
	end
	ItemsList = {};
	local FilterName = ItemDB_DBFrame_ItemList_Name:GetText();
	local FilterMinLevel = tonumber(ItemDB_DBFrame_ItemList_MinLevel:GetText());
	if (not FilterMinLevel) then
		FilterMinLevel = 0;
	end
	local FilterMaxLevel = tonumber(ItemDB_DBFrame_ItemList_MaxLevel:GetText());
	if (not FilterMaxLevel) then
		FilterMaxLevel = 70;
	end
	local cleansubclass = nil;
	local cleaninvtype = nil;
	if (ItemDB_DBFrame_selectedSubclass) then
		_, _, cleansubclass = string.find(ItemDB_DBFrame_selectedSubclass, "^"..HIGHLIGHT_FONT_COLOR_CODE.."(.*)"..FONT_COLOR_CODE_CLOSE.."$");
	end
	if (ItemDB_DBFrame_selectedInvtype) then
		_, _, cleaninvtype = string.find(ItemDB_DBFrame_selectedInvtype, "^"..HIGHLIGHT_FONT_COLOR_CODE.."(.*)"..FONT_COLOR_CODE_CLOSE.."$");
	end

	if (ItemDB_DBFrame_selectedTab == 1) then
		for i = 1, #ItemsAll do
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(ItemsAll[i]);
			itemEquipLoc = getglobal(itemEquipLoc);
			if (not itemEquipLoc or itemEquipLoc == "") then
				itemEquipLoc = ITEMDB_STRING_INVTYPE_OTHER;
			end
			if ((ItemDBRarityFilter[itemRarity + 1] == 1) and (string.find(strlower(itemName), strlower(FilterName))) and (itemMinLevel >= FilterMinLevel) and (((itemMinLevel <= FilterMaxLevel)))) then
				if (not ItemDB_DBFrame_selectedClass or (itemType == ItemDB_DBFrame_selectedClass and (not cleansubclass or (itemSubType == cleansubclass and (not cleaninvtype or (itemEquipLoc == cleaninvtype)))))) then
					local itemValue = ItemDB_DBFrame_GetValue(itemLink);
					if (not itemValue) then
						itemValue = -1;
					end
					if (not ItemDB_DBFrame_ItemList_HasValueCheckButton:GetChecked() or itemValue >= 0) then
						local itemData = {["name"] = itemName, ["link"] = itemLink, ["quality"] = itemRarity, ["level"] = itemMinLevel, ["ilevel"] = itemLevel, ["texture"] = itemTexture, ["stack"] = itemStackCount, ["value"] = itemValue}
						table.insert(ItemsList, itemData);
					end
				end
			end
		end
		ItemDB_DBFrame_ItemList_ItemCount:SetText(#ItemsList.." / "..#ItemsAll.." items");
	end
	if (ItemDB_DBFrame_selectedTab == 2) then
		for i = 1, #ItemsSession do
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(ItemsSession[i]);
			itemEquipLoc = getglobal(itemEquipLoc);
			if (not itemEquipLoc or itemEquipLoc == "") then
				itemEquipLoc = ITEMDB_STRING_INVTYPE_OTHER;
			end
			if ((ItemDBRarityFilter[itemRarity + 1] == 1) and (string.find(strlower(itemName), strlower(FilterName))) and (itemMinLevel >= FilterMinLevel) and (((itemMinLevel <= FilterMaxLevel)))) then
				if (not ItemDB_DBFrame_selectedClass or (itemType == ItemDB_DBFrame_selectedClass and (not cleansubclass or (itemSubType == cleansubclass and (not cleaninvtype or (itemEquipLoc == cleaninvtype)))))) then
					local itemValue = ItemDB_DBFrame_GetValue(itemLink);
					if (not itemValue) then
						itemValue = -1;
					end
					if (not ItemDB_DBFrame_ItemList_HasValueCheckButton:GetChecked() or itemValue >= 0) then
						local itemData = {["name"] = itemName, ["link"] = itemLink, ["quality"] = itemRarity, ["level"] = itemMinLevel, ["ilevel"] = itemLevel, ["texture"] = itemTexture, ["stack"] = itemStackCount, ["value"] = itemValue}
						table.insert(ItemsList, itemData);
					end
				end
			end
		end
		ItemDB_DBFrame_ItemList_ItemCount:SetText(#ItemsList.." / "..#ItemsSession.." items");
	end
	ItemDB_DBFrame_SortItems2();
	ItemDB_DBFrame_ItemList_Update();
end

function ItemDB_DBFrame_GetValue(itemID)
	if (IsAddOnLoaded("ItemDB_Money")) then
		return ItemDB_link2data[ItemDB_baseidFromLink(itemID)]
	end
	if (IsAddOnLoaded("KC_Items")) then
-- KC_Items < 0.94
--		return KC_Items:GetItemPrices(KC_Items:GetCode(itemID))
-- KC_Items >= 0.94
		return KC_Items.common:GetItemPrices(KC_Items.common:GetCode(itemID))
	end
	if (IsAddOnLoaded("ColaLight")) then
		return ColaLight.db.account.SellValues[ItemDB_baseidFromLink(itemID)]
	end
	if (SellValues and GetItemInfo(itemID)) then
		return SellValues[GetItemInfo(itemID)]
	end
	return nil
end

function ItemDB_DBFrame_SendChatMessage(msg, chan, lang, id)
	if not msg or not chan or not id then
		return
	end
	-- Sending a chat message will clear the AFK flag.  If the player has
	-- gone AFK we don't want to unflag them by sending a message.
	-- credits for this go to ChannelCast
	local clearAFK = GetCVar("autoClearAFK")
	SetCVar("autoClearAFK", 0)
	SendChatMessage(msg, chan, lang, id);
	SetCVar("autoClearAFK", clearAFK)
end

function ItemDB_DBFrame_DoDistChat_SendRequest()
	if (#ItemDB_DistChat_Queue_Offers > 0) then
		local offer = table.remove(ItemDB_DistChat_Queue_Offers, 1);
		local message = "[ItemDB-request] item:"..offer["baseID"];
-- -----
--		if( DEFAULT_CHAT_FRAME ) then
--			DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: Sending request:"..message);
--		end
-- -----
		if (offer["nick"] and not ItemsSessionKnown[offer["baseID"]]) then
			ItemDB_DistChat_last_Outgoing = time();
			ItemDB_DistChat_PlayersInvolved[offer["nick"]] = time();
			ItemDB_DBFrame_SendChatMessage(message, "WHISPER", nil, offer["nick"]);
		end
		return true;
	end
	return false;
end

function ItemDB_DBFrame_DoDistChat_SendAnswer()
	if (#ItemDB_DistChat_Queue_Requests > 0) then
		local request = table.remove(ItemDB_DistChat_Queue_Requests, 1);
		local itemName, itemLink, itemRarity = GetItemInfo(request["baseID"]);
		if itemName and request["nick"] then
			local message = "[ItemDB-answer] "..itemLink;
--			local message = "[ItemDB-answer] "..ITEM_QUALITY_HEXCOLORS[itemRarity].."|H"..itemLink.."|h["..itemName.."]|h|r";
--			local message = "[ItemDB-answer] "..ITEM_QUALITY_COLORS[itemRarity].hex.."|H"..itemLink.."|h["..itemName.."]|h|r";

-- -----
--			if( DEFAULT_CHAT_FRAME ) then
--				DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: Answering request:"..message);
--			end
-- -----
			ItemDB_DistChat_last_Outgoing = time();
			ItemDB_DistChat_PlayersInvolved[request["nick"]] = time();
			ItemDB_DBFrame_SendChatMessage(message, "WHISPER", nil, request["nick"]);
		end
		return true;
	end
	return false;
end

function ItemDB_DBFrame_DoDistChat_SendOffer()
	if (#ItemDB_DistChat_Queue_Offering > 0 or #ItemsSession > 0) then
		local message = "";
		if (#ItemDB_DistChat_Queue_Offering > 0) then
			local i = math.random(#ItemDB_DistChat_Queue_Offering)
			message = "[ItemDB-offer] item:"..table.remove(ItemDB_DistChat_Queue_Offering, i);
		else
			local i = math.random(#ItemsSession)
			message = "[ItemDB-offer] item:"..ItemsSession[i];
		end
-- -----
--		if( DEFAULT_CHAT_FRAME ) then
--			DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: Sending offer:"..message);
--		end
-- -----
		ItemDB_DistChat_last_Outgoing = time();
		ItemDB_DBFrame_SendChatMessage(message, "CHANNEL", nil, ItemDB_DistChat_ID);
		return true;
	end
	return false;
end

function ItemDB_DBFrame_DoDistChat()
	if ((time() - ItemDB_DistChat_last_Outgoing) > 1) then
		local msgs_sent = 0;
		local msgs_sent_offer = 0;
		if (time() - ItemDB_last_Version > 300 and ItemDB_newest_Version <= ITEMDB_DBFRAME_VERSION_NUMERIC) then
			local message = "[ItemDB-version] "..ITEMDB_DBFRAME_VERSION_NUMERIC.." "..ITEMDB_DBFRAME_VERSION;
			ItemDB_last_Version = time();
			ItemDB_DistChat_last_Outgoing = time();
			ItemDB_DBFrame_SendChatMessage(message, "CHANNEL", nil, ItemDB_DistChat_ID);
			msgs_sent = msgs_sent + 1;
		end
		if (not ItemsSession) then
			ItemDB_DBFrame_BuildItemsSession();
		end
		if (ItemDB_DistChat_msg_per_sec > 2) then
			if (ItemDB_DBFrame_DoDistChat_SendRequest()) then
				msgs_sent = msgs_sent + 1;
			end
		end
		while (msgs_sent < ItemDB_DistChat_msg_per_sec) do
			if (ItemDB_DBFrame_DoDistChat_SendAnswer()) then
				msgs_sent = msgs_sent + 1;
			else
				break;
			end
		end
		while (msgs_sent < ItemDB_DistChat_msg_per_sec) do
			if (ItemDB_DBFrame_DoDistChat_SendRequest()) then
				msgs_sent = msgs_sent + 1;
			else
				break;
			end
		end
		while (msgs_sent < ItemDB_DistChat_msg_per_sec and msgs_sent_offer < ItemDB_DistChat_msg_per_sec_offer) do
			if (ItemDB_DBFrame_DoDistChat_SendOffer()) then
				msgs_sent = msgs_sent + 1;
				msgs_sent_offer = msgs_sent_offer + 1;
			else
				break;
			end
		end
	end
end

function ItemDB_DBFrame_ProcessChat(message)
	if (not message) then
		return;
	end
	for itemID in string.gmatch(message, "|c%x+|H(item:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+)|h%[.-%]|h|r") do
-- -----
--		if( DEFAULT_CHAT_FRAME ) then
--			DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: processing chatlinked itemID "..itemID);
--		end
-- -----
		if (itemID) then
			ItemDB_DBFrame_StoreSessionItem(itemID);
		end
	end
end

function ItemDB_DBFrame_ProcessChatOffer(message, nick)
	if (not message or not nick) then
		return;
	end
-- -----
--		if( DEFAULT_CHAT_FRAME ) then
--			DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: processing offer "..message);
--		end
-- -----
	if (not ItemsSession) then
		ItemDB_DBFrame_BuildItemsSession();
	end
	local _, _, baseID = string.find(message, "%[ItemDB%-offer%] item:(%-?%d+)");
	if (baseID) then
		if (not ItemsSessionKnown[baseID]) then
			table.insert(ItemDB_DistChat_Queue_Offers, {["nick"] = nick, ["baseID"] = baseID});
		end
	end
end

function ItemDB_DBFrame_ProcessChatRequest(message, nick)
	if (not message or not nick) then
		return;
	end
-- -----
--		if( DEFAULT_CHAT_FRAME ) then
--			DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: processing request "..message);
--		end
-- -----
	if (not ItemsSession) then
		ItemDB_DBFrame_BuildItemsSession();
	end
	local _, _, baseID = string.find(message, "%[ItemDB%-request%] item:(%-?%d+)");
	if (baseID) then
		if (ItemsSessionKnown[baseID]) then
			table.insert(ItemDB_DistChat_Queue_Requests, {["nick"] = nick, ["baseID"] = baseID});
		end
	end
end

function ItemDB_DBFrame_ProcessVersion(message)
	if (not message) then
		return;
	end
	local _, _, version_num, version_str = string.find(message, "%[ItemDB%-version%] (%d+) (.+)");
	if (version_num and version_str) then
		version_num = tonumber(version_num);
		if (version_num and version_num >= ItemDB_newest_Version) then
			ItemDB_last_Version = time();
			if (version_num > ItemDB_newest_Version) then
				ItemDB_newest_Version = version_num;
-- -----
				if( DEFAULT_CHAT_FRAME ) then
					DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] "..ITEMDB_STRING_NEWVERSION.."|cffff0000"..version_str.."|r");
				end
-- -----
			end
		end
	end
end

function ItemDB_DBFrame_Inspect(unit)
	if not (UnitExists(unit) and UnitIsPlayer(unit) and UnitIsFriend(unit, "player")) then
--	if not CanInspect(unit) then
		return
	end
	local timenow = time();
	if (timenow - ItemDB_Inspected_cleaned > 5) then -- clean up cache at most every 5 seconds
		for key, value in pairs(ItemDB_Inspected) do
			if (timenow - value > 300) then -- remove after 5min
				ItemDB_Inspected[key] = NIL;
			end
		end
		ItemDB_Inspected_cleaned = timenow;
	end
	local name = UnitName(unit);
	if not CheckInteractDistance(unit, 1) or ItemDB_Inspected[name] then
		return
	end
	ItemDB_Inspected[name] = timenow;
-- -----
	if( ITEMDB_DEBUG_ENABLE and ITEMDB_DEBUG_INSPECT and DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: |cffffff00inspecting player |cff0000ff"..name);
	end
-- -----
	for i=1, #ItemDB_InventorySlotName do
		local itemlink = GetInventoryItemLink(unit, GetInventorySlotInfo(ItemDB_InventorySlotName[i]));
		if (itemlink) then
			local _, _, itemID = string.find(itemlink, "|c%x+|H(item:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+)|h%[.-%]|h|r");
			ItemDB_DBFrame_StoreSessionItem(itemID);
		end
	end
end

function ItemDB_DBFrame_ScanAuction()
-- -----
	if( ITEMDB_DEBUG_ENABLE and DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: Scanning auction...");
	end
-- -----
	local numBatchAuctions = GetNumAuctionItems("list");
	for auctionid = 1, numBatchAuctions do
		local itemlink = GetAuctionItemLink("list", auctionid);
		if (itemlink) then
			local _, _, itemID = string.find(itemlink, "|c%x+|H(item:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+)|h%[.-%]|h|r");
			ItemDB_DBFrame_StoreSessionItem(itemID);
		end
	end
end

function ItemDB_DBFrame_ScanBags()
-- -----
	if( ITEMDB_DEBUG_ENABLE and DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: Scanning bags...");
	end
-- -----
	for bagid = 0, NUM_BAG_SLOTS do
		local size = GetContainerNumSlots(bagid);
		if (size) then
			for slotid = 1, size do
				local itemlink = GetContainerItemLink(bagid, slotid);
				if (itemlink) then
					local _, _, itemID = string.find(itemlink, "|c%x+|H(item:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+)|h%[.-%]|h|r");
					ItemDB_DBFrame_StoreSessionItem(itemID);
				end
			end
		end
	end
end

function ItemDB_DBFrame_ScanBank()
-- -----
	if( ITEMDB_DEBUG_ENABLE and DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: Scanning bank...");
	end
-- -----
	for bagid_i = NUM_BAG_SLOTS, (NUM_BAG_SLOTS + NUM_BANKBAGSLOTS) do
		local bagid = bagid_i;
		if (bagid == NUM_BAG_SLOTS) then
			bagid = BANK_CONTAINER;
		end
		local size = GetContainerNumSlots(bagid);
		if (size) then
			for slotid = 1, size do
				local itemlink = GetContainerItemLink(bagid, slotid);
				if (itemlink) then
					local _, _, itemID = string.find(itemlink, "|c%x+|H(item:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+)|h%[.-%]|h|r");
					ItemDB_DBFrame_StoreSessionItem(itemID);
				end
			end
		end
	end
end

function ItemDB_DBFrame_ScanMerchant()
-- -----
	if( ITEMDB_DEBUG_ENABLE and DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: Scanning Merchant...");
	end
-- -----
	if (MerchantFrame:IsVisible() and MerchantFrame.selectedTab == 1 and ItemDB_last_FrameScan > ItemDB_last_FrameScan_blocktime) then
		-- This prevents further update events from being handled if we're already processing one.
		-- This is done to prevent the game from freezing under certain conditions.
		ItemDB_last_FrameScan = time();
		local numMerchantItems = GetMerchantNumItems();
		for i = 1, numMerchantItems do
			local itemlink = GetMerchantItemLink(i);
			if (itemlink) then
-- -----
--		if( DEFAULT_CHAT_FRAME ) then
--			DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: link: "..itemlink.." - "..string.gsub(itemlink, "|", "/"));
--		end
-- -----
				local _, _, itemID = string.find(itemlink, "|c%x+|H(item:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+)|h%[.-%]|h|r");
				ItemDB_DBFrame_StoreSessionItem(itemID);
			end
		end
	end
end

function ItemDB_DBFrame_ScanTradeSkill()
-- -----
	if( ITEMDB_DEBUG_ENABLE and DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: Scanning tradeskill...");
	end
-- -----
	if (TradeSkillFrame ~= nil and TradeSkillFrame:IsVisible() and (time() - ItemDB_last_FrameScan > ItemDB_last_FrameScan_blocktime)) then
		-- This prevents further update events from being handled if we're already processing one.
		-- This is done to prevent the game from freezing under certain conditions.
		ItemDB_last_FrameScan = time();
		for i=1, GetNumTradeSkills() do
			local _, type, _, _ = GetTradeSkillInfo(i);
			if (type ~= "header") then
				local itemlink = GetTradeSkillItemLink(i);
				if (itemlink) then
					local _, _, itemID = string.find(itemlink, "|c%x+|H(item:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+)|h%[.-%]|h|r");
					ItemDB_DBFrame_StoreSessionItem(itemID);
				end
				for j=1, GetTradeSkillNumReagents(i) do
					local reagentName, _, _, _ = GetTradeSkillReagentInfo(i, j);
					if (reagentName) then
						local itemlink = GetTradeSkillReagentItemLink(i, j);
						if (itemlink) then
							local _, _, itemID = string.find(itemlink, "|c%x+|H(item:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+)|h%[.-%]|h|r");
							ItemDB_DBFrame_StoreSessionItem(itemID);
						end
					end
				end
			end
		end
	end
end

function ItemDB_DBFrame_ScanCraft()
-- -----
	if( ITEMDB_DEBUG_ENABLE and DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: Scanning crafts...");
	end
-- -----
	if (CraftFrame ~= nil and CraftFrame:IsVisible() and (time() - ItemDB_last_FrameScan > ItemDB_last_FrameScan_blocktime)) then
		-- This prevents further update events from being handled if we're already processing one.
		-- This is done to prevent the game from freezing under certain conditions.
		ItemDB_last_FrameScan = time();
		for i=1, GetNumCrafts() do
			local _, type, _, _ = GetCraftInfo(i);
			if (type ~= "header") then
				local itemlink = GetCraftItemLink(i);
				if (itemlink) then
					local _, _, itemID = string.find(itemlink, "|c%x+|H(item:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+)|h%[.-%]|h|r");
					ItemDB_DBFrame_StoreSessionItem(itemID);
				end
				for j=1, GetCraftNumReagents(i) do
					local reagentName, _, _, _ = GetCraftReagentInfo(i, j);
					if (reagentName) then
						local itemlink = GetCraftReagentItemLink(i, j);
						if (itemlink) then
							local _, _, itemID = string.find(itemlink, "|c%x+|H(item:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+)|h%[.-%]|h|r");
							ItemDB_DBFrame_StoreSessionItem(itemID);
						end
					end
				end
			end
		end
	end
end

function ItemDB_DBFrame_StoreSessionItem(itemID)
	if (not itemID) then
		return false;
	end
	if (not ItemsSession) then
		ItemDB_DBFrame_BuildItemsSession();
	end
	baseID = ItemDB_baseidFromLink(itemID);
	if (not ItemsSessionKnown[baseID]) then
		ItemDB_DBFrame_CacheTooltip:SetHyperlink(itemID);
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc = GetItemInfo(baseID);
		if ( itemName ) then	-- valid item
			itemEquipLoc = getglobal(itemEquipLoc);
			if (not itemEquipLoc or itemEquipLoc == "") then
				itemEquipLoc = ITEMDB_STRING_INVTYPE_OTHER;
			end
			table.insert(ItemsSession, baseID);
			ItemsSessionKnown[baseID] = "";
			if (not types or not types[itemType] or not types[itemType][itemSubType] or not types[itemType][itemSubType][itemEquipLoc]) then
				ItemDB_DBFrame_BuildItemsAll();
			end
			table.insert(ItemDB_DistChat_Queue_Offering, baseID);
-- -----
			if( ITEMDB_DEBUG_ENABLE and ITEMDB_DEBUG_SESSIONITEMS and DEFAULT_CHAT_FRAME ) then
				DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: stored itemID "..itemID.." -> "..fixNIL(ItemDB_DBFrame_ItemButton_GetLinkFromID(itemID)));
			end
-- -----
			return true;
		else
-- -----
			if( ITEMDB_DEBUG_ENABLE and ITEMDB_DEBUG_SESSIONITEMS and DEFAULT_CHAT_FRAME ) then
				DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: itemID "..itemID.." is not stored yet but doesn't return data... will recheck later...");
			end
-- -----
			ItemsSessionRecheck[baseID] = "";
		end
	end
	return false;
end

function ItemDB_DBFrame_RecheckSessionItems()
	local checkitems = {}
	for baseid, _ in pairs(ItemsSessionRecheck) do
		table.insert(checkitems, baseid)
	end
	for i=1, #checkitems do
		if (ItemDB_DBFrame_StoreSessionItem("item:"..checkitems[i]..":0:0:0:0:0:0:0")) then
			ItemsSessionRecheck[checkitems[i]] = nil;
		end
	end
end

function ItemDB_DBFrame_ResetButton_OnClick()
	ItemDB_DBFrame_ItemList_Name:SetText("");
	ItemDB_DBFrame_ItemList_MinLevel:SetText("");
	ItemDB_DBFrame_ItemList_MaxLevel:SetText("");
	ItemDBRarityFilter = {1, 1, 1, 1, 1, 1, 1};
	ItemDB_DBFrame_ItemList_HasValueCheckButton:SetChecked(false);
	ItemDB_DBFrame_selectedClass = nil;
	ItemDB_DBFrame_selectedSubclass = nil;
	ItemDB_DBFrame_selectedInvtype = nil;
	ItemDB_DBFrame_Filters_Update();
	ItemDB_DBFrame_BuildItemsList();
end

-- Dropdown functions
function ItemDB_DBFrame_ItemList_RarityFilterDropDown_OnLoad()
	UIDropDownMenu_Initialize(this, ItemDB_DBFrame_ItemList_RarityFilterDropDown_Initialize);
	UIDropDownMenu_SetText(FILTER, this);
	UIDropDownMenu_SetWidth(130, ItemDB_DBFrame_ItemList_RarityFilterDropDown);
end

function ItemDB_DBFrame_ItemList_RarityFilterDropDown_Initialize()
	for i=0, #ITEM_QUALITY_COLORS  do
		local info = {};
		local checked = nil;
		if (ItemDBRarityFilter[i + 1] == 1) then
			checked = 1;
		end
		local color = ITEM_QUALITY_COLORS[i];
		info.text = getglobal("ITEM_QUALITY"..i.."_DESC");
		info.textR = color.r;
		info.textG = color.g;
		info.textB = color.b;
		info.value = i;
		info.checked = checked;
		info.keepShownOnClick = 1;
		info.func = ItemDB_DBFrame_ItemList_RarityFilterDropDown_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

function ItemDB_DBFrame_ItemList_RarityFilterDropDown_OnClick()
	if ( UIDropDownMenuButton_GetChecked() ) then
		ItemDBRarityFilter[this.value + 1] = 0;
	else
		ItemDBRarityFilter[this.value + 1] = 1;
	end
end

function ItemDB_DBFrame_ItemList_Update()
	local numItemsList = #ItemsList;
	local button, buttonName, iconTexture, itemName, color, moneyFrame, buttonHighlight;
	local offset = FauxScrollFrame_GetOffset(ItemDB_DBFrame_ItemList_BrowseScrollFrame);
--	local index;
--	local isLastSlotEmpty;
	local name, quality, level, texture, value, count;
	-- Update sort arrows
	ItemDB_DBFrame_SortButton_UpdateArrow(ItemDB_DBFrame_ItemList_BrowseQualitySort, "quality");
	ItemDB_DBFrame_SortButton_UpdateArrow(ItemDB_DBFrame_ItemList_BrowseNameSort, "name");
	ItemDB_DBFrame_SortButton_UpdateArrow(ItemDB_DBFrame_ItemList_BrowseLevelSort, "level");
	ItemDB_DBFrame_SortButton_UpdateArrow(ItemDB_DBFrame_ItemList_BrowseValueSort, "value");

	-- Show the no results text if no items found
	if ( numItemsList == 0 ) then
		ItemDB_DBFrame_ItemList_NoResultsText:Show();
	else
		ItemDB_DBFrame_ItemList_NoResultsText:Hide();
	end

	for i=1, ITEMDB_NUM_ITEMS_TO_DISPLAY do
		index = offset + i;
		button = getglobal("ItemDB_DBFrame_ItemList_ItemButton"..i);
		-- Show or hide auction buttons
		if ( index > numItemsList ) then
			button:Hide();
			-- If the last button is empty then set isLastSlotEmpty var
			if ( i == ITEMDB_NUM_ITEMS_TO_DISPLAY ) then
				isLastSlotEmpty = 1;
			end
		else
			button:Show();

			buttonName = "ItemDB_DBFrame_ItemList_ItemButton"..i;
			name = ItemsList[index]["name"];
			quality = ItemsList[index]["quality"];
			level = ItemsList[index]["level"];
			texture = ItemsList[index]["texture"];
			count = ItemsList[index]["stack"];
			value = ItemsList[index]["value"];

			if (not texture or texture == "") then
				texture = "Interface\\Icons\\INV_Misc_QuestionMark";
			end

			-- Resize button if there isn't a scrollbar
			buttonHighlight = getglobal("ItemDB_DBFrame_ItemList_ItemButton"..i.."Highlight");
			if ( numItemsList <= ITEMDB_NUM_ITEMS_TO_DISPLAY ) then
				button:SetWidth(625);
				buttonHighlight:SetWidth(589);
				ItemDB_DBFrame_ItemList_BrowseValueSort:SetWidth(207);
			else
				button:SetWidth(600);
				buttonHighlight:SetWidth(562);
				ItemDB_DBFrame_ItemList_BrowseValueSort:SetWidth(184);
			end
			-- Set name and quality color
			color = ITEM_QUALITY_COLORS[quality];
			itemName = getglobal(buttonName.."Name");
			itemName:SetText(name);
			itemName:SetVertexColor(color.r, color.g, color.b);
			-- Set level
			if ( level > UnitLevel("player") ) then
				getglobal(buttonName.."Level"):SetText(RED_FONT_COLOR_CODE..level..FONT_COLOR_CODE_CLOSE);
			else
				getglobal(buttonName.."Level"):SetText(level);
			end
			-- Set item texture, count, and usability
			iconTexture = getglobal(buttonName.."ItemIconTexture");
			iconTexture:SetTexture(texture);
--			if ( not canUse ) then
--				iconTexture:SetVertexColor(1.0, 0.1, 0.1);
--			else
--				iconTexture:SetVertexColor(1.0, 1.0, 1.0);
--			end
			itemCount = getglobal(buttonName.."ItemCount");
			if ( count > 1 ) then
				itemCount:SetText(count);
				itemCount:Show();
			else
				itemCount:Hide();
			end
			-- Set item's value
			moneyFrame = getglobal(buttonName.."MoneyFrame");
			unsellableText = getglobal(buttonName.."UnsellableText");
			-- If not bidAmount set the bid amount to the min bid
			if ( value and value > 0 ) then
				MoneyFrame_Update(moneyFrame:GetName(), value);
				moneyFrame:SetPoint("RIGHT", buttonName, "RIGHT", 10, 3);
				moneyFrame:Show();
				unsellableText:Hide();
			elseif ( value and value == 0 ) then
				MoneyFrame_Update(moneyFrame:GetName(), 0);
				moneyFrame:Hide();
				unsellableText:Show();
			else
				MoneyFrame_Update(moneyFrame:GetName(), 0);
				moneyFrame:Hide();
				unsellableText:Hide();
			end

			-- Set highlight
--			if ( GetSelectedAuctionItem("list") and (offset + i) == GetSelectedAuctionItem("list") ) then
--				button:LockHighlight();
				
--			else
--				button:UnlockHighlight();
--			end
		end
	end
	FauxScrollFrame_Update(ItemDB_DBFrame_ItemList_BrowseScrollFrame, numItemsList, ITEMDB_NUM_ITEMS_TO_DISPLAY, ITEMDB_ITEM_BUTTON_HEIGHT);
end

-- SortButton functions
function ItemDB_DBFrame_SortButton_UpdateArrow(button, type, sort)
	if ( IsAuctionSortReversed(type, sort) ) then
		getglobal(button:GetName().."Arrow"):SetTexCoord(0, 0.5625, 1.0, 0);
	else
		getglobal(button:GetName().."Arrow"):SetTexCoord(0, 0.5625, 0, 1.0);
	end
end

function ItemDB_DBFrame_ItemButton_OnEnter(index)
	GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
	GameTooltip:SetHyperlink(ItemsList[index]["link"]);
	if ( IsControlKeyDown() ) then
		ShowInspectCursor();
	else
		ResetCursor();
	end
end

function ItemDB_DBFrame_Link2Chat(index)
	if (ItemDB_DBFrame_selectedTab == 2 or ItemsSessionKnown[ItemDB_baseidFromLink(ItemsList[index]["link"])]) then
		ChatEdit_InsertLink(ItemDB_DBFrame_ItemButton_GetLink(index));
	else
		ItemDB_DBFrame_clickedButton = index;
		StaticPopup_Show("ITEMDB_LINKING_WARNING");
	end
end

function ItemDB_DBFrame_ItemButton_GetLink(index)
	if (ItemsList[index]) then
--		local color = ITEM_QUALITY_COLORS[ItemsList[index]["quality"]];
--		local colorstring = string.format("|cff%02x%02x%02x", color.r * 255, color.g * 255, color.b * 255);
--		return colorstring.."|H"..ItemsList[index]["link"].."|h["..ItemsList[index]["name"].."]|h|r";
		return ItemsList[index]["link"];
--		return ITEM_QUALITY_HEXCOLORS[ItemsList[index]["quality"]].."|H"..ItemsList[index]["link"].."|h["..ItemsList[index]["name"].."]|h|r";
--		return ITEM_QUALITY_COLORS[ItemsList[index]["quality"]].hex.."|H"..ItemsList[index]["link"].."|h["..ItemsList[index]["name"].."]|h|r";
	end
end

function ItemDB_DBFrame_ItemButton_GetItemID(index)
	return ItemsList[index]["link"];
end

function ItemDB_DBFrame_ItemButton_GetLinkFromID(itemID)
	local itemName, itemLink, itemRarity = GetItemInfo(itemID);
	if (itemName) then
		return itemLink;
--		return ITEM_QUALITY_HEXCOLORS[itemRarity].."|H"..itemLink.."|h["..itemName.."]|h|r";
	end
	return nil;
end

function ItemDB_DBFrame_SortButton_UpdateArrow(button, sort)
	if ( sort == ItemDBSortCurrent ) then
		getglobal(button:GetName().."Arrow"):Show();
	else
		getglobal(button:GetName().."Arrow"):Hide();
		return;
	end
	if ( ItemDBSortReverse == 1 ) then
		getglobal(button:GetName().."Arrow"):SetTexCoord(0, 0.5625, 1.0, 0);
	else
		getglobal(button:GetName().."Arrow"):SetTexCoord(0, 0.5625, 0, 1.0);
	end
end

function ItemDB_DBFrame_SortItems(sort)
	if ( sort == ItemDBSortCurrent ) then
		ItemDBSortReverse = 1 - ItemDBSortReverse;
	else
		ItemDBSortCurrent = sort;
	end
	ItemDB_DBFrame_SortButton_UpdateArrow(ItemDB_DBFrame_ItemList_BrowseQualitySort, "quality");
	ItemDB_DBFrame_SortButton_UpdateArrow(ItemDB_DBFrame_ItemList_BrowseNameSort, "name");
	ItemDB_DBFrame_SortButton_UpdateArrow(ItemDB_DBFrame_ItemList_BrowseLevelSort, "level");
	ItemDB_DBFrame_SortButton_UpdateArrow(ItemDB_DBFrame_ItemList_BrowseValueSort, "value");
	ItemDB_DBFrame_SortItems2();
	ItemDB_DBFrame_ItemList_Update();
end

function ItemDB_DBFrame_SortItems2()
	if (ItemDBSortCurrent == "quality") then
		table.sort(ItemsList, ItemDB_DBFrame_SortItems_CompareQuality);
	end
	if (ItemDBSortCurrent == "name") then
		table.sort(ItemsList, ItemDB_DBFrame_SortItems_CompareName);
	end
	if (ItemDBSortCurrent == "level") then
		table.sort(ItemsList, ItemDB_DBFrame_SortItems_CompareLevel);
	end
	if (ItemDBSortCurrent == "value") then
		table.sort(ItemsList, ItemDB_DBFrame_SortItems_CompareValue);
	end
end

function ItemDB_DBFrame_SortItems_CompareQuality(a, b)
	if (not a or not b) then
		return true;
	end
	if ((ItemDBSortReverse == 0 and a["quality"] < b["quality"]) or (ItemDBSortReverse == 1 and a["quality"] > b["quality"])) then
		return true;
	end
	if ((ItemDBSortReverse == 0 and a["quality"] > b["quality"]) or (ItemDBSortReverse == 1 and a["quality"] < b["quality"])) then
		return false;
	end
	return ItemDB_DBFrame_SortItems_CompareName2(a, b);
end

function ItemDB_DBFrame_SortItems_CompareName(a, b)
	if (not a or not b) then
		return true;
	end
	if ((ItemDBSortReverse == 0 and a["name"] < b["name"]) or (ItemDBSortReverse == 1 and a["name"] > b["name"])) then
		return true;
	else
		return false;
	end
end

function ItemDB_DBFrame_SortItems_CompareName2(a, b)
	if (not a or not b) then
		return true;
	end
	if (a["name"] < b["name"]) then
		return true;
	else
		return false;
	end
end

function ItemDB_DBFrame_SortItems_CompareLevel(a, b)
	if (not a or not b) then
		return true;
	end
	if ((ItemDBSortReverse == 0 and a["level"] < b["level"]) or (ItemDBSortReverse == 1 and a["level"] > b["level"])) then
		return true;
	end
	if ((ItemDBSortReverse == 0 and a["level"] > b["level"]) or (ItemDBSortReverse == 1 and a["level"] < b["level"])) then
		return false;
	end
	return ItemDB_DBFrame_SortItems_CompareName2(a, b);
end

function ItemDB_DBFrame_SortItems_CompareValue(a, b)
	if (not a or not b) then
		return true;
	end
	if ((ItemDBSortReverse == 0 and a["value"] == nil) or (ItemDBSortReverse == 1 and b["value"] == nil)) then
		return true;
	end
	if ((ItemDBSortReverse == 0 and b["value"] == nil) or (ItemDBSortReverse == 1 and a["value"] == nil)) then
		return false;
	end
	if ((ItemDBSortReverse == 0 and a["value"] < b["value"]) or (ItemDBSortReverse == 1 and a["value"] > b["value"])) then
		return true;
	end
	if ((ItemDBSortReverse == 0 and a["value"] > b["value"]) or (ItemDBSortReverse == 1 and a["value"] < b["value"])) then
		return false;
	end
	return ItemDB_DBFrame_SortItems_CompareName2(a, b);
end

function ItemDB_DBFrame_Filters_Update()
	ItemDB_DBFrame_Filters_UpdateClasses();
	-- Update scrollFrame
	FauxScrollFrame_Update(ItemDB_DBFrame_FilterScrollFrame, #ITEMDB_OPEN_FILTER_LIST, ITEMDB_NUM_FILTERS_TO_DISPLAY, ITEMDB_ITEM_FILTER_HEIGHT);
end

function ItemDB_DBFrame_Filters_UpdateClasses()
	-- Initialize the list of open filters
	ITEMDB_OPEN_FILTER_LIST = {};
	for i=1, #ItemTypes do
		if ( ItemDB_DBFrame_selectedClass and ItemDB_DBFrame_selectedClass == ItemTypes[i] ) then
			tinsert(ITEMDB_OPEN_FILTER_LIST, {ItemTypes[i], "class", i, 1});
			ItemDB_DBFrame_Filters_UpdateSubClasses(ItemSubTypes[ItemDB_DBFrame_selectedClass]);
		else
			tinsert(ITEMDB_OPEN_FILTER_LIST, {ItemTypes[i], "class", i, nil});
		end
	end
	
	-- Display the list of open filters
	local button, index, info, isLast;
	local offset = FauxScrollFrame_GetOffset(ItemDB_DBFrame_FilterScrollFrame);
	for i=1, ITEMDB_NUM_FILTERS_TO_DISPLAY do
		button = getglobal("ItemDB_DBFrame_FilterButton"..i);
		if ( #ITEMDB_OPEN_FILTER_LIST > ITEMDB_NUM_FILTERS_TO_DISPLAY ) then
			button:SetWidth(136);
		else
			button:SetWidth(156);
		end
		index = offset + i;
		if ( index <= #ITEMDB_OPEN_FILTER_LIST ) then
			info = ITEMDB_OPEN_FILTER_LIST[index];
			ItemDB_DBFrame_FilterButton_SetType(button, info[2], info[1], info[5]);
			button.index = info[3];
			if ( info[4] ) then
				button:LockHighlight();
			else
				button:UnlockHighlight();
			end
			button:Show();
		else
			button:Hide();
		end
	end
end

function ItemDB_DBFrame_Filters_UpdateSubClasses(subtypes)
	if (not subtypes) then
		return;
	end
	local subClass;
	for i=1, #subtypes do
		subClass = HIGHLIGHT_FONT_COLOR_CODE..subtypes[i]..FONT_COLOR_CODE_CLOSE; 
		if ( ItemDB_DBFrame_selectedSubclass and ItemDB_DBFrame_selectedSubclass == subClass ) then
			tinsert(ITEMDB_OPEN_FILTER_LIST, {subtypes[i], "subclass", i, 1});
			local cleansubclass = nil;
			if (ItemDB_DBFrame_selectedSubclass) then
				_, _, cleansubclass = string.find(ItemDB_DBFrame_selectedSubclass, "^"..HIGHLIGHT_FONT_COLOR_CODE.."(.*)"..FONT_COLOR_CODE_CLOSE.."$");
			end
			if (ItemInvTypes[ItemDB_DBFrame_selectedClass]) then
				ItemDB_DBFrame_Filters_UpdateInvTypes(ItemInvTypes[ItemDB_DBFrame_selectedClass][cleansubclass]);
			end
		else
			tinsert(ITEMDB_OPEN_FILTER_LIST, {subtypes[i], "subclass", i, nil});
		end
	end
end

function ItemDB_DBFrame_Filters_UpdateInvTypes(invtypes)
	if (not invtypes) then
		return;
	end
	local invType, isLast;
	for i=1, #invtypes do
		invType = HIGHLIGHT_FONT_COLOR_CODE..invtypes[i]..FONT_COLOR_CODE_CLOSE; 
		if ( i == #invtypes ) then
			isLast = 1;
		end
		if ( ItemDB_DBFrame_selectedInvtype and ItemDB_DBFrame_selectedInvtype == invType ) then
			tinsert(ITEMDB_OPEN_FILTER_LIST, {invtypes[i], "invtype", i, 1, isLast});
		else
			tinsert(ITEMDB_OPEN_FILTER_LIST, {invtypes[i], "invtype", i, nil, isLast});
		end
	end
end

function ItemDB_DBFrame_FilterButton_SetType(button, type, text, isLast)
	local normalText = getglobal(button:GetName().."NormalText");
	local normalTexture = getglobal(button:GetName().."NormalTexture");
	local line = getglobal(button:GetName().."Lines");
	if ( type == "class" ) then
		button:SetText(text);
		normalText:SetPoint("LEFT", button, "LEFT", 4, 0);
		normalTexture:SetAlpha(1.0);	
		line:Hide();
	elseif ( type == "subclass" ) then
		button:SetText(HIGHLIGHT_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE);
		normalText:SetPoint("LEFT", button, "LEFT", 12, 0);
		normalTexture:SetAlpha(0.4);
		line:Hide();
	elseif ( type == "invtype" ) then
		button:SetText(HIGHLIGHT_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE);
		normalText:SetPoint("LEFT", button, "LEFT", 20, 0);
		normalTexture:SetAlpha(0.0);	
		if ( isLast ) then
			line:SetTexCoord(0.4375, 0.875, 0, 0.625);
		else
			line:SetTexCoord(0, 0.4375, 0, 0.625);
		end
		line:Show();
	end
	button.type = type; 
end

function ItemDB_DBFrame_Filter_OnClick()
	if ( this.type == "class" ) then
		if ( ItemDB_DBFrame_selectedClass == this:GetText() ) then
			ItemDB_DBFrame_selectedClass = nil;
--			ItemDB_DBFrame_selectedClassIndex = nil;
		else
			ItemDB_DBFrame_selectedClass = this:GetText();
--			ItemDB_DBFrame_selectedClassIndex = this.index;
		end
		ItemDB_DBFrame_selectedSubclass = nil;
--		ItemDB_DBFrame_selectedSubclassIndex = nil;
--		ItemDB_DBFrame_selectedInvtype = nil;
--		ItemDB_DBFrame_selectedInvtypeIndex = nil;
	elseif ( this.type == "subclass" ) then
		if ( ItemDB_DBFrame_selectedSubclass == this:GetText() ) then
			ItemDB_DBFrame_selectedSubclass = nil;
--			ItemDB_DBFrame_selectedSubclassIndex = nil;
		else
			ItemDB_DBFrame_selectedSubclass = this:GetText();
--			ItemDB_DBFrame_selectedSubclassIndex = this.index;
		end
		ItemDB_DBFrame_selectedInvtype = nil;
--		ItemDB_DBFrame_selectedInvtypeIndex = nil;
	elseif ( this.type == "invtype" ) then
		if ( ItemDB_DBFrame_selectedInvtype == this:GetText() ) then
			ItemDB_DBFrame_selectedInvtype = nil;
		else
			ItemDB_DBFrame_selectedInvtype = this:GetText();
--			ItemDB_DBFrame_selectedInvtypeIndex = this.index;
		end
	end
	ItemDB_DBFrame_Filters_Update()
end

function fixNIL(a)
	if (not a) then
		return "NIL";
	end
	return a;
end

function ItemDB_DistChat_Join()
	local name, id;
	local i = 0;
	repeat
		name = ItemDB_DistChat_Base..i;
--		tid, tname = JoinChannelByName(name, "", DEFAULT_CHAT_FRAME:GetID());
		JoinChannelByName(name);
		id = GetChannelName(name);
-- -----
		if( ITEMDB_DEBUG_ENABLE and DEFAULT_CHAT_FRAME ) then
			DEFAULT_CHAT_FRAME:AddMessage("[ItemDB] Debug: i: "..i.." - id: "..id.." - name: "..name);
		end
-- -----
		i = i + 1;
	until (id > 0 or i >= 100);
	if (id > 0) then
		ItemDB_DistChat_Name = name;
		ItemDB_DistChat_ID = id;
--		ChatFrame_RemoveChannel(ChatFrame1, name);
		ItemDB_last_Version = time();
	end;
end

function ItemDB_DistChat_Leave()
	if (ItemDB_DistChat_Name) then
		LeaveChannelByName(ItemDB_DistChat_Name);
	end
	ItemDB_DistChat_Name = nil;
	ItemDB_DistChat_ID = nil;
end
